@@ -1,6 +1,6 @@
 package Math::Vector::Real::kdTree;
 
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 
 use 5.010;
 use strict;
@@ -9,6 +9,7 @@ use Carp;
 
 use Math::Vector::Real;
 use Sort::Key::Top qw(nkeypartref nhead ntail nkeyhead);
+use Hash::Util::FieldHash qw(idhash);
 
 our $max_per_pole = 12;
 our $recommended_per_pole = 6;
@@ -405,8 +406,104 @@ sub _find_nearest_vector_all_internal {
     }
 }
 
+sub find_two_nearest_vectors {
+    my $self = shift;
+    my $t = $self->{tree} or return;
+    my $vs = $self->{vs};
+    my $d2 = 'inf' + 0;
+    if (my ($rix0, $rix1, $rd2) = _find_two_nearest_vectors($vs, $t, $d2)) {
+        return wantarray ? ($rix0, $rix1, sqrt($rd2)) : sqrt($rd2)
+    }
+    ()
+}
+
+sub _pole_id {
+    my ($id, $deep) = __pole_id(@_);
+    "$id/$deep";
+}
+
+sub __pole_id {
+    my ($vs, $t) = @_;
+    if (defined $t->[_axis]) {
+        my ($id, $deep) = __pole_id($vs, $t->[_s0]);
+        return ($id, $deep+1);
+    }
+    return ($t->[_ixs][0], 0)
+}
+
+sub _find_two_nearest_vectors {
+    my ($vs, $t, $best_d2) = @_;
+
+    my @best_ixs = (undef, undef);
+
+    my @queue_t1 = $t;
+    my @queue_t2 = undef;
+    my @queue_d2 = undef;
+
+    while (@queue_t1) {
+        my $t1 = pop @queue_t1;
+        my $t2 = pop @queue_t2;
+        my $d2 = pop @queue_d2;
+        if ($t2) {
+            if ($d2 < $best_d2) {
+                unless (defined $t1->[_axis]) {
+                    unless (defined $t2->[_axis]) {
+                        for my $ix1 (@{$t1->[_ixs]}) {
+                            my $v1 = $vs->[$ix1];
+                            for my $ix2 (@{$t2->[_ixs]}) {
+                                my $d2 = Math::Vector::Real::dist2($v1, $vs->[$ix2]);
+                                if ($d2 < $best_d2) {
+                                    $best_d2 = $d2;
+                                    @best_ixs = ($ix1, $ix2);
+                                }
+                            }
+                        }
+                        next;
+                    }
+                    ($t1, $t2) = ($t2, $t1);
+                }
+                for my $s (@{$t1}[_s0, _s1]) {
+                    my $d2 = Math::Vector::Real->dist2_between_boxes(@{$s}[_c0, _c1], @{$t2}[_c0, _c1]);
+                    if ($d2 < $best_d2) {
+                        unshift @queue_t1, $t2;
+                        unshift @queue_t2, $s;
+                        unshift @queue_d2, $d2;
+                    }
+                }
+            }
+        }
+        else {
+            if (defined $t1->[_axis]) {
+                my ($s0, $s1) = @{$t1}[_s0, _s1];
+                push @queue_t1, $s0, $s0, $s1;
+                push @queue_t2, $s1, undef, undef;
+                push @queue_d2, 0, 0, 0;
+            }
+            else {
+                my $ixs = $t1->[_ixs];
+                for my $i (1 .. $#$ixs) {
+                    my $ix1 = $ixs->[$i];
+                    my $v1 = $vs->[$ix1];
+                    for my $j (0 .. $i - 1) {
+                        my $ix2 = $ixs->[$j];
+                        my $d2 = Math::Vector::Real::dist2($v1, $vs->[$ix2]);
+                        if ($d2 < $best_d2) {
+                            $best_d2 = $d2;
+                            @best_ixs = ($ix1, $ix2);
+                        }
+                    }
+                }
+            }
+        }
+    }
+    (@best_ixs, $best_d2)
+}
+
 sub find_in_ball {
     my ($self, $z, $d, $but) = @_;
+    if (defined $but and ref $but ne 'HASH') {
+        $but = { $but => 1 };
+    }
     _find_in_ball($self->{vs}, $self->{tree}, $z, $d * $d, $but);
 }
 
@@ -432,13 +529,16 @@ sub _find_in_ball {
                         ? grep { !$but->{$_} and $vs->[$_]->dist2($z) <= $d2 } @$ixs
                         : grep { $vs->[$_]->dist2($z) <= $d2 } @$ixs );
             }
-        }
 
-        $t = pop @queue or last;
+            $t = pop @queue or last;
+        }
     }
 
     if (wantarray) {
-        return ($but ? grep !$but->{$_}, @r : @r);
+        if ($but) {
+            return grep !$but->{$_}, @r;
+        }
+        return @r;
     }
     return $r;
 }
@@ -447,7 +547,7 @@ sub find_farthest_vector {
     my ($self, $v, $d, @but) = @_;
     my $t = $self->{tree} or return;
     my $vs = $self->{vs};
-    my $d2 = ($d ? $d * $d : 0);
+    my $d2 = ($d ? $d * $d : -1);
     my $but;
     if (@but) {
         if (@but == 1 and ref $but[0] eq 'HASH') {
@@ -511,15 +611,40 @@ sub _find_farthest_vector {
     }
 }
 
-sub k_means_start {
+sub find_random_vector {
+    my $self = shift;
+    my $t = $self->{tree} or return;
+    my $vs = $self->{vs};
+    my $hidden = $self->{hidden};
+    if (not $hidden or @$vs > 20 * keys(%$hidden)) {
+        # pick directly when the hidden elements are less than 5% of the total
+        while (1) {
+            my $ix = int rand @$vs;
+            return $ix unless $hidden and $hidden->{$ix};
+        }
+    }
+    _find_random_vector($vs, $t);
+}
+
+sub _find_random_vector {
+    my ($vs, $t) = @_;
+    while (defined $t->[_axis]) {
+        $t = $t->[rand($t->[_n]) < $t->[_s0][_n] ? _s0 : _s1];
+    }
+    $t->[_ixs][rand $t->[_n]]
+}
+
+sub k_means_seed {
     my ($self, $n_req) = @_;
     $n_req = int($n_req) or return;
     my $t = $self->{tree} or return;
     my $vs = $self->{vs};
-    _k_means_start($vs, $t, $n_req);
+    _k_means_seed($vs, $t, $n_req);
 }
 
-sub _k_means_start {
+*k_means_start = \&k_means_seed;
+
+sub _k_means_seed {
     my ($vs, $t, $n_req) = @_;
     if ($n_req <= 1) {
         return if $n_req < 1;
@@ -534,8 +659,8 @@ sub _k_means_start {
             my $n1 = $s1->[_n];
             my $n0_req = int(0.5 + $n_req * ($n0 / $n));
             $n0_req = $n0 if $n0_req > $n0;
-            return (_k_means_start($vs, $s0, $n0_req),
-                    _k_means_start($vs, $s1, $n_req - $n0_req));
+            return (_k_means_seed($vs, $s0, $n0_req),
+                    _k_means_seed($vs, $s1, $n_req - $n0_req));
         }
         else {
             my $ixs = $t->[_ixs];
@@ -551,6 +676,153 @@ sub _k_means_start {
     }
 }
 
+our $k_means_seed_pp_test;
+
+sub _k_means_seed_pp_test {
+    my ($self, $err, $kms, $players, $weights) = @_;
+    my @w;
+    my $last = 0;
+    for my $i (0..$#$players) {
+        my $p = $players->[$i];
+        my $w = $weights->[$i] - $last;
+        $last = $weights->[$i];
+
+        my @store;
+        if (ref $p) {
+            _push_all($p, \@store);
+        }
+        else {
+            @store = $p
+        }
+        if (@store) {
+            $w /= @store;
+            $w[$_] = $w for @store;
+        }
+    }
+    my $vs = $self->{vs};
+    $w[$_] //= 0 for 0..$#$vs;
+
+    $k_means_seed_pp_test->($self, $err, [map $self->{vs}[$_], @$kms], \@w);
+}
+
+sub k_means_seed_pp {
+    my ($self, $n_req, $err) = @_;
+    $n_req = int($n_req) or return;
+    $err ||= 0.5;
+    my $t = $self->{tree} or return;
+    my $vs = $self->{vs};
+    my $km = $self->find_random_vector;
+
+    my (@km, @d2);
+    idhash my %extra; # [$min_d2, $max_d2]
+
+    # my (@player, @weight, @queue);
+    # $#player = @$vs; # preallocate memory
+
+    my (@weight, @queue);
+    $#weight = @$vs; # preallocate memory
+
+    while (1) {
+        push @km, $km;
+        last unless @km < $n_req;
+
+        # update distances
+        @queue = $t;
+        while (my $p = pop @queue) {
+            my $kmv = $vs->[$km];
+            my ($c0, $c1) = @{$p}[_c0, _c1];
+            my $extra = $extra{$p} //= ['inf', 'inf'];
+            my ($min_d2, $max_d2) = @$extra;
+            my $min_d2_to_box = $kmv->dist2_to_box($c0, $c1);
+            if ($max_d2 > $min_d2_to_box) {
+                if (defined $p->[_axis]) {
+                    push @queue, @{$p}[_s0, _s1];
+                }
+                else {
+                    for (@{$p->[_ixs]}) {
+                        my $d2 = $kmv->dist2($vs->[$_]);
+                        if ($d2 < ($d2[$_] //= $d2)) {
+                            $d2[$_] = $d2;
+                        }
+                    }
+                }
+
+                if ($min_d2_to_box < $min_d2) {
+                    $extra->[0] = $min_d2_to_box;
+                }
+
+                my $max_d2_to_box = $kmv->max_dist2_to_box($c0, $c1);
+                if ($max_d2_to_box < $max_d2) {
+                    $extra->[1] = $max_d2_to_box;
+                }
+            }
+        }
+
+        # find players and weight them
+        my $weight = 0;
+        # @player = ();
+        @weight = ();
+
+        # @queue = $t;
+        # while (my $p = pop @queue) {
+        #     my $extra = $extra{$p} or die "internal error: extra information missing for $p";
+        #     my ($min_d2, $max_d2) = @$extra;
+
+        #     if ($max_d2 * $err < $min_d2) {
+        #         $weight += $p->[_n] * ($min_d2 + $max_d2) * 0.5;
+        #         push @weight, $weight;
+        #         push @player, $p;
+        #     }
+        #     else {
+        #         if (defined $p->[_axis]) {
+        #             push @queue, @{$p}[_s0, _s1];
+        #         }
+        #         else {
+        #             for (@{$p->[_ixs]}) {
+        #                 if (my $d2 = $d2[$_]) {
+        #                     $weight += $d2;
+        #                     push @weight, $weight;
+        #                     push @player, $_;
+        #                 }
+        #             }
+        #         }
+        #     }
+        # }
+
+        for my $ix (0..@$vs) {
+            $weight += $d2[$ix] // 0;
+            $weight[$ix] += $weight;
+        }
+
+        # in order to check the algorithm we have to tap it here
+        # $k_means_seed_pp_test and @km > 1 and
+        #    $self->_k_means_seed_pp_test($err, \@km, \@player, \@weight);
+
+        # to many k-means requested?
+        # @player or last;
+
+        # select a position on the weight queue:
+        my $dice = rand($weight);
+
+        # and use binary search to look for it:
+        my $i = 0;
+        my $j = @weight;
+        while ($i < $j) {
+            my $pivot = (($i + $j) >> 1);
+            if ($weight[$pivot] < $dice) {
+                $i = $pivot + 1;
+            }
+            else {
+                $j = $pivot;
+           }
+        }
+        #my $player = $player[$i];
+        #$km = (ref $player ? _find_random_vector($vs, $player) : $player);
+        $km = $i;
+    }
+    return @{$vs}[@km];
+}
+
 sub k_means_loop {
     my ($self, @k) = @_;
     @k or next;
@@ -713,22 +985,30 @@ sub _ordered_by_proximity {
 sub _dump_to_string {
     my ($vs, $t, $indent, $opts) = @_;
     my ($n, $c0, $c1, $sum) = @{$t}[_n, _c0, _c1, _sum];
+    my $id = ($opts->{pole_id} ? _pole_id($vs, $t)." " : '');
     if (defined (my $axis = $t->[_axis])) {
         my ($s0, $s1, $cut) = @{$t}[_s0, _s1, _cut];
-        return ( "${indent}n: $n, c0: $c0, c1: $c1, sum: $sum, axis: $axis, cut: $cut\n" .
+        return ( "${indent}${id}n: $n, c0: $c0, c1: $c1, sum: $sum, axis: $axis, cut: $cut\n" .
                  _dump_to_string($vs, $s0, "$indent$opts->{tab}", $opts) .
                  _dump_to_string($vs, $s1, "$indent$opts->{tab}", $opts) );
     }
     else {
-        my $o = ( "${indent}n: $n, c0: $c0, c1: $c1, sum: $sum\n" .
+        my $remark = $opts->{remark} // [];
+        my $o = ( "${indent}${id}n: $n, c0: $c0, c1: $c1, sum: $sum\n" .
                   "${indent}$opts->{tab}ixs: [" );
-        if ($opts->{dump_vectors} // 1) {
-            $o .= join(", ", map "$_ $vs->[$_]", @{$t->[_ixs]});
+        my @str;
+        for my $ix (@{$t->[_ixs]}) {
+            my $colored_ix = (@$remark and grep($ix == $_, @$remark)
+                              ? Term::ANSIColor::colored($ix, 'red')
+                              : $ix);
+            if ($opts->{dump_vectors} // 1) {
+                push @str, "$colored_ix $vs->[$ix]";
+            }
+            else {
+                push @str, $colored_ix;
+            }
         }
-        else {
-            $o .= join(", ", @{$t->[_ixs]});
-        }
-        return $o . "]\n";
+        return $o . join(', ', @str) . "]\n";
     }
 }
 
@@ -740,6 +1020,7 @@ sub dump_to_string {
     my $hidden = join ", ", keys %{$self->{hidden} || {}};
     my $o = "tree: n: $nvs, hidden: {$hidden}\n";
     if (my $t = $self->{tree}) {
+        require Term::ANSIColor if $opts{remark};
         return $o . _dump_to_string($vs, $t, $tab, \%opts);
     }
     else {
@@ -854,7 +1135,15 @@ for the farthest point.
 Given the index of a point on the tree this method returns the index
 of the farthest vector also from the tree.
 
-=item @k = $t->k_means_start($n)
+=item ($ix0, $ix1, $d) = $t->find_two_nearest_vectors
+
+This method returns the indexes of two vectors from the three such
+that the distance between them is minimal. The distance is returned as
+the third output value.
+
+In scalar context, just the distance is returned.
+
+=item @k = $t->k_means_seed($n)
 
 This method uses the internal tree structure to generate a set of
 point that can be used as seeds for other C<k_means> methods.
@@ -907,7 +1196,7 @@ The module can be used to calculate the k-means of a set of vectors as follows:
   
   # k-mean calculation
   my $t = Math::Vector::Real::kdTree->new(@v);
-  my @means = $t->k_means_start($k);
+  my @means = $t->k_means_seed($k);
   @means = $t->k_means_loop(@means);
   @assign = $t->k_means_assign(@means);
   my @cluster = map [], 1..$k;
@@ -930,7 +1219,7 @@ L<Math::Vector::Real>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2011-2014 by Salvador FandiE<ntilde>o E<lt>sfandino@yahoo.comE<gt>
+Copyright (C) 2011-2015 by Salvador FandiE<ntilde>o E<lt>sfandino@yahoo.comE<gt>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.12.3 or,
